/**
 * - Digitalis Internal Framework v2.0 -
 *
 * (C) 2007, Digitalis Informatica. 
 * 
 * Distribuicao e Gestao de Informatica, Lda.
 * Estrada de Paco de Arcos num.9 - Piso -1
 * 2780-666 Paco de Arcos
 * Telefone: (351) 21 4408990
 * Fax: (351) 21 4408999
 * http://www.digitalis.pt 
 */
package pt.digitalis.utils.config;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

/**
 * Utility class to keep a Config field/property key and accessor methods for reflection manipulation purposes
 * 
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Sep 27, 2007
 * 
 */
public class ConfigItem {

    /** The property key */
    private String key;

    /** The property getter method */
    private Method getter;

    /** The property setter method */
    private Method setter;

    /** The constructor for the setter argument */
    private Constructor<?> constructor;

    /** The default value for the item */
    private String defaultValue;

    /** The class of the item */
    private Class<?> itemClazz;

    /**
     * Default Constructor
     * 
     * @param key
     *            the property key
     * @param getter
     *            the property getter method
     * @param setter
     *            the property setter method
     * @param constructor
     *            the constructor for the value conversion of the setter value
     * @param defaultValue
     *            the default value for the item
     * @param clazz
     *            the item class
     */
    public ConfigItem(String key, Method getter, Method setter, Constructor<?> constructor, String defaultValue,
            Class<?> clazz) {
        this.key = key;
        this.getter = getter;
        this.setter = setter;
        this.constructor = constructor;
        this.defaultValue = defaultValue;
        this.itemClazz = clazz;
    }

    /**
     * @return the itemClazz
     */
    public Class<?> getItemClass() {
        return itemClazz;
    }

    /**
     * @return the getter
     */
    public Method getGetter() {
        return getter;
    }

    /**
     * @return the key
     */
    public String getKey() {
        return key;
    }

    /**
     * @return the setter
     */
    public Method getSetter() {
        return setter;
    }

    /**
     * @return the constructor
     */
    public Constructor<?> getConstructor() {
        return constructor;
    }

    /**
     * @return the defaultValue
     */
    public String getDefaultValue() {
        return defaultValue;
    }
}
